# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2013 Michael Bethencourt michaelpb@gmail.com
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from gi.repository import Gtk # pylint: disable=E0611
from gi.repository.GdkPixbuf import Pixbuf # pylint: disable=E0611

from fsgamer_lib.helpers import get_builder

import gettext
import fsgamer_app as fsg_app
from gettext import gettext as _
gettext.textdomain('fsgamer')

class ChooseDialog(Gtk.Dialog):
    __gtype_name__ = "ChooseDialog"

    def __new__(cls):
        """Special static method that's automatically called by Python when 
        constructing a new instance of this class.
        
        Returns a fully instantiated ChooseDialog object.
        """
        builder = get_builder('ChooseDialog')
        new_object = builder.get_object('choose_dialog')
        new_object.finish_initializing(builder)
        return new_object

    def finish_initializing(self, builder):
        """Called when we're finished initializing.

        finish_initalizing should be called after parsing the ui definition
        and creating a ChooseDialog object with it in order to
        finish initializing the start of the new ChooseDialog
        instance.
        """
        # Get a reference to the builder and set up the signals.
        self.builder = builder
        self.ui = builder.get_ui(self)
        self.desktop_entry_list = fsg_app.DesktopEntryList()
        self.ui.checkbox_show_games.set_active(True)
        self.populate_desktop_entry_list()
        self.choice = None

    def populate_desktop_entry_list(self):
        games_only = self.ui.checkbox_show_games.get_active()
        self.choice = None
        liststore = Gtk.ListStore(Pixbuf, str)
        self.liststore = liststore
        self.ui.iconview.set_model(liststore)
        self.ui.iconview.set_pixbuf_column(0)
        self.ui.iconview.set_text_column(1)

        DEFAULT = 'gtk-file'

        self.app_list = self.desktop_entry_list
        if games_only:
            self.app_list = self.app_list.filter_games()

        icon_theme = Gtk.IconTheme.get_default()
        for xdg_info in self.app_list:
            icon = xdg_info.getIcon()
            name = xdg_info.getName()
            try:
                pixbuf = icon_theme.load_icon(icon, 48, 0)
            except:
                pixbuf = icon_theme.load_icon(DEFAULT, 48, 0)
            self.liststore.append([pixbuf, name])

    def on_btn_ok_clicked(self, widget, data=None):
        """The user has elected to save the changes.

        Called before the dialog returns Gtk.ResponseType.OK from run().
        """
        self.choice = self.get_choice()
        self.hide()

    def on_btn_cancel_clicked(self, widget, data=None):
        """The user has elected cancel changes.

        Called before the dialog returns Gtk.ResponseType.CANCEL for run()
        """
        self.choice = None
        self.hide()

    def on_checkbox_show_games_toggled(self, widget):
        self.populate_desktop_entry_list()

    def get_app(self):
        any_selected = self.ui.iconview.get_selected_items()
        if any_selected:
            indices = any_selected[0].get_indices()
            if indices:
                return self.app_list[indices[0]]
        return None

    def icon_selection_changed(self, widget, data=None):
        app = self.get_app()
        if app:
            self.ui.command_entry.set_editable(False)
            self.ui.command_entry.set_has_frame(False)
            txt = app.getExec() or app.getPath()
            self.ui.command_entry.set_text(txt)
        else:
            self.ui.command_entry.set_editable(True)
            self.ui.command_entry.set_has_frame(True)
            self.choice = self.get_app()
            txt = self.ui.command_entry.get_text()
            self.choice = fsg_app.Application(command=txt)

    def get_choice(self):
        app = self.get_app()
        if app:
            return fsg_app.Application(desktop=app)
        else:
            txt = self.ui.command_entry.get_text()
            return fsg_app.Application(command=txt)

if __name__ == "__main__":
    dialog = ChooseDialog()
    dialog.show()
    Gtk.main()
