import glib
import dbus
from dbus.mainloop.glib import DBusGMainLoop
import utils
from pipes import quote

#ESPEAK_ENABLED = False
#try:
#    import espeak
#    ESPEAK_ENABLED = True
#except ImportError:
#    pass

def listen(callback):
    def notifications(bus, message):
        if message.get_member() == "Notify":
            callback(message.get_args_list(), bus)

    DBusGMainLoop(set_as_default=True)

    bus = dbus.SessionBus()
    bus.add_match_string_non_blocking("interface='org.freedesktop.Notifications'")
    bus.add_message_filter(notifications)

    mainloop = glib.MainLoop()
    mainloop.run()


ESPEAK_PATH = utils.shell_run.which("espeak")
PAPLAY_PATH = utils.shell_run.which("paplay")


def parse_args(args):
    return {
        'summary': unicode(args[3]),
        'body': unicode(args[4]),
    }

TEMPLATE = "Note. %(summary)s. %(body)s."

class callbacks:
    @staticmethod
    def espeak(args, bus):
        if not ESPEAK_PATH:
            return

        saying = "Note. %(summary)s %(body)s" % parse_args(args)
        command = "espeak %s --stdout | paplay" % quote(saying)
        utils.shell_run(command)

if not ESPEAK_PATH:
    utils.warning("ESPEAK is not installed. (sudo apt-get install espeak)")
elif not PAPLAY_PATH:
    utils.warning("PAPLAY is not installed. (sudo apt-get install paplay)")

def main():
    if ESPEAK_PATH:
        listen(callbacks.espeak)
    else:
        utils.warning("Not working, espeak not installed. (sudo apt-get install espeak)")


