import xdg.Menu
import xdg.DesktopEntry as d
import xdg.BaseDirectory
import xdg.Menu
import locale
import subprocess
import os

import utils
############################################
# Monkey patch to address an encoding bug in pyxdg (fixed in current trunk)
# thanks to launchpad.net/~strycore for telling me how to fix this :)
locale._strxfrm = locale.strxfrm
def strxfrm(s):
    return locale._strxfrm(s.encode('utf-8'))
locale.strxfrm = strxfrm
############################################

def load_de(fn):
    try:
        new_entry = d.DesktopEntry(filename=fn)
    except:
        # Catch all exception just to keep things rolling incase one of
        # these is invalid in a way I cannot even imagine
        print("Uh oh, could not load ", fn)
    return new_entry


XDG_MENU_PATH = '/etc/xdg/menus'
def get_xdg_applications():
    lst = []

    def add_menu_to_list(menu, depth = 0):
        for entry in menu.getEntries():
            if isinstance(entry, xdg.Menu.Menu):
                add_menu_to_list(entry, depth)
            elif isinstance(entry, xdg.Menu.MenuEntry):
                lst.append((menu.getPath(), entry.DesktopFileID, entry.DesktopEntry.getFileName()))

    # Try the various ways to add the menu
    try:
        # sometimes this breaks, do to xdg bug perhaps?
        menu = xdg.Menu.parse()
    except Exception as e:
        # In an error, try to add all instead
        utils.warning("Xdg Menu not found. Something is wonky with your DE or "+
                        "distro. FSGamer might not be able to detect installed "+
                        "applications.")

        for dirpath, dirnames, filenames in os.walk(XDG_MENU_PATH):
            for filename in filenames:
                path = os.path.join(dirpath, filename)
                try:
                    menu = xdg.Menu.parse(path)
                except Exception as e:
                    # Give up at this point
                    utils.warning("Parsing error in %s" % path)
                else:
                    add_menu_to_list(menu)
    else:
        add_menu_to_list(menu)

    if not lst:
        utils.warning("FSGamer was not able to detect any applications installed!")

    desktop_entries = []
    for menu_path, fileid, fn in lst:
        if fn.endswith(".fsgamer.desktop"):
            # One of our own, skip
            continue

        new_entry = load_de(fn)
        if new_entry.getName().lower().strip() == 'fsgamer':
            # It's probably me!
            continue

        new_entry.filename = fn
        desktop_entries.append(new_entry)
    return desktop_entries


class DesktopEntryList(list):
    def __init__(self, initial=None):
        entries = initial or get_xdg_applications()
        list.__init__(self, entries)

    def filter_games(self):
        def game_filter(item):
            l = [i.lower() for i in item.getCategories()]
            return any(map(lambda s: 'game' in s,  l))
        return DesktopEntryList(filter(game_filter, self))

    def __repr__(self):
        s = u''
        for item in self:
            s += u'%s - Icon(%s) - Categories(%s)\n' % (repr(item.getName()), repr(item.getIcon()), repr(item.getCategories()))
        return s


